Attribute VB_Name = "Modul1"
' trppt2svg
'
' Export a Powerpoint slide to a Scalable Vector Graphics (SVG) file
'
'
' This software is Open Source. Use it under the GNU Public License.
' Please do read file COPYING.TXT that you shall have received along with this code
'
' Copyright 2019-2023 Dr. Thomas Redelberger, redetho@gmx.de
' Repository: http://web222.webclient5.de/prj/content/trppt2svg
'
'
' * Credits
'
' This code is based on code svg.vba from TIM Pagden, released 2002-08-11
'
' $Id: trppt2svg.bas 1.3 2023/05/18 15:59:43 redetho Exp redetho $


Option Explicit

'' Configure the software

#Const MY_DEBUG = False

#Const USE_ABS_VIEW_PORT = False
#Const USE_BACKGROUND = False


'' Hardwired output attributes

' coordinates output precision
Rem Const mSnglRndg As String = "0.##"  ' up to two decimal places
Const mSnglRndg As String = "0.#"   ' up to one decimal place
Rem Const mSnglRndg As String = "0"     ' no fractions


' Default indent string at beginning of line: two spaces
Const myItN As Long = 2

' Hardwired SVG dash array, in user co-ordinates
Const myDashArrayStr = "5,5"


'' Make code more readable and efficient (?)

Const myC As String = ";"   ' separator char for style attributes
Const myQ As String = """"  ' ASCII quotation mark "


' Friendly name of this macro
Const globModName As String = "Export to SVG"



'' Enums for ADO Stream, as I do not reference a type lib

' Enum for LineSeparator
Const adCR = 13
Const adCRLF = -1       ' default
Const adLF = 10         ' Unix tradition

' Enum for Mode
Const adModeRead = 1
Const adModeWrite = 2
Const adModeReadWrite = 3
' There are many more modes...

' Enum for Type
Const adTypeBinary = 1
Const adTypeText = 2

' Enum for SaveOptions
Const adSaveCreateNotExist = 1  ' will fail if the file exists
Const adSaveCreateOverWrite = 2

' Enum for WriteText
Const adWriteChar = 0
Const adWriteLine = 1   ' must set the LineSeparator property, acc. to MS docu



' Global objects

Dim globErrText As String

Dim strmOb As Object
Dim globFSOb As Object

Dim markersDone As Boolean  ' used to write out markers only once



'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Auxiliary functions
'


Private Function rgb_to_colour_hash(RGBlng As Long) As String
''  converts VBA RGB integer to #rrggbb format string
Rem "#" & Right("00000" & Hex(RGBlng), 6)  does not work because of wrong byte order
    rgb_to_colour_hash = "#" _
    & Right("0" & Hex(RGBlng Mod 256), 2) _
    & Right("0" & Hex(RGBlng \ 256 Mod 256), 2) _
    & Right("0" & Hex(RGBlng \ 65536 Mod 256), 2)
End Function    ' rgb_to_colour_hash



Private Function myDecFmt(ByVal nmbr As Single) As String
' 0, 1 or 2 decimals after the decimal /dot/ i.e. not localised

#If MY_DEBUG Then
    If nmbr < -10000 Then
        myDecFmt = "UNDRFL"
        Exit Function
    End If
    If nmbr > 10000 Then
        myDecFmt = "OVERFL"
        Exit Function
    End If
#End If

    myDecFmt = Format(nmbr, mSnglRndg)      ' Format outputs the /locale/ decimal separator
    myDecFmt = Replace(myDecFmt, ",", ".")  ' ensure it is a dot all the time
    ' remove a lonely trailing dot
    If Right(myDecFmt, 1) = "." Then myDecFmt = Left(myDecFmt, Len(myDecFmt) - 1)

End Function    ' myDecFmt



Private Function normaliseText(s As String) As String
    normaliseText = Replace(s, vbVerticalTab, "")       ' remove vertical tab, whose purpose in PPT I do not know
    normaliseText = Replace(normaliseText, vbCr, "")    ' remove carriage return. It seems to indicate a new paragraph in PPT
    normaliseText = Replace(normaliseText, "&", "&amp;")    ' XML escape
    normaliseText = Replace(normaliseText, "<", "&lt;")     ' XML escape
    normaliseText = Replace(normaliseText, ">", "&gt;")     ' XML escape
End Function    ' normaliseText




'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Low level functions
'


Private Function get_points(ByRef s As Shape, ByVal isClosed As Boolean) As String
    Dim numNodes As Long
    Dim i As Long
    
    If isClosed Then
        numNodes = s.Nodes.Count - 1
    Else
        numNodes = s.Nodes.Count
    End If
    
    get_points = myDecFmt(s.Vertices(1, 1)) & "," & myDecFmt(s.Vertices(1, 2))
    
    ''  build remaining points
    For i = 2 To numNodes
      get_points = get_points & " " & myDecFmt(s.Vertices(i, 1)) & "," & myDecFmt(s.Vertices(i, 2))
    Next  ' Node
    
End Function    ' get_points



Private Function fillStrokeStyle(ByRef s As Shape) As String

    fillStrokeStyle = " style="""

    If s.Fill.Visible = msoFalse Then
        fillStrokeStyle = fillStrokeStyle & "fill:none;"
    Else
        fillStrokeStyle = fillStrokeStyle & "fill:" & rgb_to_colour_hash(s.Fill.ForeColor.RGB) & myC
        If s.Fill.Transparency > 0 Then
            fillStrokeStyle = fillStrokeStyle & "fill-opacity:" & myDecFmt(1 - s.Fill.Transparency) & myC
        End If
    End If

    If s.Line.Visible = msoFalse Then
        fillStrokeStyle = fillStrokeStyle & "stroke:none;"
    Else
        fillStrokeStyle = fillStrokeStyle & "stroke:" & rgb_to_colour_hash(s.Line.ForeColor.RGB) & myC
        fillStrokeStyle = fillStrokeStyle & "stroke-width:" & myDecFmt(s.Line.Weight) & myC
        If s.Line.DashStyle <> msoLineSolid Then
            fillStrokeStyle = fillStrokeStyle & "stroke-dasharray:" & myDashArrayStr & myC
        End If
    End If
    
    fillStrokeStyle = fillStrokeStyle & myQ
End Function    ' fillStrokeStyle



Private Function get_svg_line_points(ByRef s As Shape) As String
    Dim x1 As Single
    Dim y1 As Single
    Dim x2 As Single
    Dim y2 As Single
    
    If s.HorizontalFlip Then
        If s.VerticalFlip Then ''  both flips
            x1 = s.Left + s.Width
            y1 = s.Top + s.Height
            x2 = s.Left
            y2 = s.Top
        Else ''  horizontal flip only
            x1 = s.Left + s.Width
            y1 = s.Top
            x2 = s.Left
            y2 = s.Top + s.Height
        End If
    Else ''  vertical flip only
        If s.VerticalFlip Then
            x1 = s.Left
            y1 = s.Top + s.Height
            x2 = s.Left + s.Width
            y2 = s.Top
        Else ''  no flip of any kind
            x1 = s.Left
            y1 = s.Top
            x2 = s.Left + s.Width
            y2 = s.Top + s.Height
        End If
    End If
    
    get_svg_line_points = _
            "x1=""" & myDecFmt(x1) _
       & """ y1=""" & myDecFmt(y1) _
       & """ x2=""" & myDecFmt(x2) _
       & """ y2=""" & myDecFmt(y2) _
       & """"
End Function    ' get_svg_line_points



Private Function get_svg_rect_points(ByRef s As Shape) As String
    
    get_svg_rect_points = _
            "x=""" & myDecFmt(s.Left) _
       & """ y=""" & myDecFmt(s.Top) _
       & """ width=""" & myDecFmt(s.Width) _
       & """ height=""" & myDecFmt(s.Height) _
       & """"
End Function    ' get_svg_rect_points



Private Function get_svg_ellipse_points(ByRef s As Shape) As String
    Dim x As Single
    Dim y As Single
    Dim w As Single
    Dim h As Single

    x = s.Left
    y = s.Top
    w = s.Width
    h = s.Height
    get_svg_ellipse_points = _
           "cx=""" & myDecFmt(x + (w / 2)) _
      & """ cy=""" & myDecFmt(y + (h / 2)) _
      & """ rx=""" & myDecFmt(w / 2) _
      & """ ry=""" & myDecFmt(h / 2) _
      & """"
End Function    ' get_svg_ellipse_points



Private Function get_svg_path_points(ByRef s As Shape) As String
    Dim numNodes As Long
    Dim i As Long
    Dim j As Long
  
    numNodes = s.Nodes.Count

    get_svg_path_points = "d=""M" & myDecFmt(s.Vertices(1, 1)) & "," & myDecFmt(s.Vertices(1, 2))
    j = 1
    
    For i = 2 To numNodes
        If s.Nodes(i).SegmentType = msoSegmentLine Then
            get_svg_path_points = get_svg_path_points & " L" & myDecFmt(s.Vertices(i, 1)) & "," & myDecFmt(s.Vertices(i, 2))
            j = 1
        Else    ' msoSegmentCurve
            If (j Mod 3) = 1 Then
                get_svg_path_points = get_svg_path_points & " C" & myDecFmt(s.Vertices(i, 1)) & "," & myDecFmt(s.Vertices(i, 2))
            Else
                get_svg_path_points = get_svg_path_points & " " & myDecFmt(s.Vertices(i, 1)) & "," & myDecFmt(s.Vertices(i, 2))
            End If
            j = j + 1
        End If
    Next i

    get_svg_path_points = get_svg_path_points & myQ
End Function    ' get_svg_path_points



Private Function line_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim markersNeeded As Boolean
    Dim marker_start As String
    Dim marker_end As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If
   
    If s.Line.BeginArrowheadStyle = msoArrowheadOval Then
        marker_start = " marker-start=""url(#Circle)"""
        markersNeeded = True
    End If
    If s.Line.BeginArrowheadStyle = msoArrowheadTriangle Then
        marker_start = " marker-start=""url(#TriangleS)"""
        markersNeeded = True
    End If
    If s.Line.EndArrowheadStyle = msoArrowheadOval Then
        marker_end = " marker-end=""url(#Circle)"""
        markersNeeded = True
    End If
    If s.Line.EndArrowheadStyle = msoArrowheadTriangle Then
        marker_end = " marker-end=""url(#TriangleE)"""
        markersNeeded = True
    End If
    line_to_svg = dbgStr _
     & "<line " & get_svg_line_points(s) _
     & fillStrokeStyle(s) _
     & marker_start & marker_end & "/>"
     
    If markersNeeded And Not markersDone Then
        outputDefs
        markersDone = True
    End If
End Function    ' line_to_svg



Private Function textStyle(ByRef t As TextRange, ByVal noInhrtBl As Boolean) As String
    ' noInhrtBl = True allows to be explicit, i.e. to avoid that a tspan inherits
    ' from the parent text element
    
    Const myBLOscl As Single = 1#   ' empirical scaling factor for base line offset
    Const myFontScl As Single = 0.666666    ' PPT scales the character size this down
    Dim ffText As String
    Dim dy As Single
    Dim fontScl
    
    If t.Font.Subscript Or t.Font.Superscript Then
        dy = -myBLOscl * t.Font.BaselineOffset * t.Font.Size
        textStyle = " transform=""translate(0 " & myDecFmt(dy) & ")"" style="""
        fontScl = myFontScl
    Else
        textStyle = " style="""
        fontScl = 1#
    End If
    
    ''  get font family *** To be checked. Should add ;serif or ;sans-serif
    ffText = t.Font.Name
    textStyle = textStyle & "font-family:" & ffText & myC
    
    ''  get font size. Some user agents seem to need the "px" unit
    textStyle = textStyle & "font-size:" & myDecFmt(fontScl * t.Font.Size) & "px;"
    
    ''  get colour of font
    textStyle = textStyle & "fill:" & rgb_to_colour_hash(t.Font.Color.RGB) & myC
    
    '' get font attributes
    If t.Font.Italic Then
      textStyle = textStyle & "font-style:italic;"
    Else
        If noInhrtBl Then textStyle = textStyle & "font-style:normal;"
    End If
      
    If t.Font.Bold Then
      textStyle = textStyle & "font-weight:bold;"
    Else
        If noInhrtBl Then textStyle = textStyle & "font-weight:normal;"
    End If
    
    If t.Font.Underline Then
      textStyle = textStyle & "text-decoration:underline;"
    Else
        If noInhrtBl Then textStyle = textStyle & "text-decoration:none;"
    End If
    
    textStyle = textStyle & myQ

End Function    ' textStyle



Private Function buildParagraph(orient As Long, firstPar As Boolean, parTextRange As TextRange) As String
    Dim numLines As Long
    Dim numRuns As Long
    Dim i As Long
    Dim j As Long
    Dim dy As String
 
    Select Case orient
    Case msoTextOrientationHorizontal
        dy = myDecFmt(parTextRange.Lines(1).BoundHeight)
    Case msoTextOrientationUpward, msoTextOrientationDownward
        dy = myDecFmt(parTextRange.Lines(1).BoundWidth)
    End Select
    
    '' First Line
    
    If firstPar Then    ' First paragraph is in PPT always without "Durchschuss"
        ' first paragraph, first line, first Run
        buildParagraph = normaliseText(parTextRange.Lines(1).Runs(1))
    Else
        ' first Run needs to start at beg. of line and set the correct y
        buildParagraph = _
         "<tspan x=""0"" dy = """ & dy & myQ _
         & textStyle(parTextRange.Lines(1).Runs(1), True) _
         & ">" _
         & normaliseText(parTextRange.Lines(1).Runs(1)) _
         & "</tspan>"
    End If

    numRuns = parTextRange.Lines(1).Runs.Count
    For j = 2 To numRuns    ' second and further runs
        buildParagraph = buildParagraph & _
        "<tspan" _
        & textStyle(parTextRange.Lines(1).Runs(j), True) _
        & ">" _
        & normaliseText(parTextRange.Lines(1).Runs(j)) _
        & "</tspan>"
    Next j

    
    '' Second and Further Lines

    numLines = parTextRange.Lines.Count
    
    For i = 2 To numLines       ' check if there are more lines in /this/ paragraph
        Select Case orient
        Case msoTextOrientationHorizontal
            dy = myDecFmt(parTextRange.Lines(i).BoundHeight)
        Case msoTextOrientationUpward, msoTextOrientationDownward
            dy = myDecFmt(parTextRange.Lines(i).BoundWidth)
        End Select
    
        ' first Run needs to start at beg. of line and set the correct y
        buildParagraph = buildParagraph _
         & "<tspan x=""0"" dy = """ & dy & myQ _
         & textStyle(parTextRange.Lines(i).Runs(1), True) _
         & ">" _
         & normaliseText(parTextRange.Lines(i).Runs(1)) _
         & "</tspan>"
        
        numRuns = parTextRange.Lines(i).Runs.Count
        For j = 2 To numRuns    ' second and further runs
            buildParagraph = buildParagraph & _
            "<tspan" _
            & textStyle(parTextRange.Lines(i).Runs(j), True) _
            & ">" _
            & normaliseText(parTextRange.Lines(i).Runs(j)) _
            & "</tspan>"
        Next j
    Next i

End Function    ' buildParagraph




Private Function textbox_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    ' BoundHeight - myBLestim * FontSize  is my crude proxy for baseline distance from top
    Const myBLestim As Single = 0.25
    
    Dim firstParTextRange As TextRange
    Dim trfAttr As String       ' transform attribute
    Dim numParagraphs As Long
    Dim j As Long
    Dim x As String
    Dim y As String
    Dim w As String
    Dim h As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If
   
    
    numParagraphs = s.TextFrame.TextRange.Paragraphs.Count
    
    Set firstParTextRange = s.TextFrame.TextRange.Paragraphs(1, 1)  ' deal with first paragraph

    Select Case s.TextFrame.Orientation
    Case msoTextOrientationHorizontal
        x = myDecFmt(s.TextFrame.TextRange.BoundLeft)
        ' Crude Approximation of the baseline height
        y = myDecFmt(s.TextFrame.TextRange.BoundTop + firstParTextRange.Lines(1).BoundHeight - myBLestim * firstParTextRange.Font.Size)
        trfAttr = " transform=""translate(" & x & " " & y & ")"""
    Case msoTextOrientationUpward
        x = myDecFmt(s.TextFrame.TextRange.BoundLeft + firstParTextRange.Font.Size)
        y = myDecFmt(s.TextFrame.TextRange.BoundTop + s.TextFrame.TextRange.BoundHeight)
        trfAttr = " transform=""translate(" & x & " " & y & ") rotate(-90)"""
    Case msoTextOrientationDownward
        x = myDecFmt(s.TextFrame.TextRange.BoundLeft + s.TextFrame.TextRange.BoundWidth - firstParTextRange.Font.Size)
        y = myDecFmt(s.TextFrame.TextRange.BoundTop)
        trfAttr = " transform=""translate(" & x & " " & y & ") rotate(90)"""
    End Select
    w = myDecFmt(s.TextFrame.TextRange.BoundWidth)
    h = myDecFmt(s.TextFrame.TextRange.BoundHeight)
    

    ' first paragraph
    textbox_to_svg = dbgStr _
     & "<text xml:space=""preserve""" & trfAttr _
     & " width=""" & w _
     & """ height=""" & h & myQ _
     & textStyle(firstParTextRange.Runs(1), False) _
     & ">" & buildParagraph(s.TextFrame.Orientation, True, firstParTextRange)
    
    For j = 2 To numParagraphs      ' do possible further paragraphs
        textbox_to_svg = textbox_to_svg & buildParagraph(s.TextFrame.Orientation, False, s.TextFrame.TextRange.Paragraphs(j, 1))
    Next j
    
    textbox_to_svg = textbox_to_svg & "</text>"

End Function    ' textbox_to_svg



Private Function curve_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim markersNeeded As Boolean
    Dim marker_start As String
    Dim marker_end As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " (curve_to_svg) -->" & vbLf & indentStr
#End If

    Select Case s.Line.BeginArrowheadStyle
    Case msoArrowheadOval
        marker_start = " marker-start=""url(#Circle)"""
        markersNeeded = True
    Case msoArrowheadTriangle
        marker_start = " marker-start=""url(#TriangleS)"""
        markersNeeded = True
    End Select

    Select Case s.Line.EndArrowheadStyle
    Case msoArrowheadOval
        marker_end = " marker-end=""url(#Circle)"""
        markersNeeded = True
    Case msoArrowheadTriangle
        marker_end = " marker-end=""url(#TriangleE)"""
        markersNeeded = True
    End Select
    
    curve_to_svg = dbgStr _
     & "<path " _
     & get_svg_path_points(s) _
     & fillStrokeStyle(s) _
     & marker_start & marker_end & "/>"
     
    If markersNeeded And Not markersDone Then
        outputDefs
        markersDone = True
    End If
End Function    ' curve_to_svg




Private Function polyline_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim markersNeeded As Boolean
    Dim marker_start As String
    Dim marker_end As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " (polyline_to_svg) -->" & vbLf & indentStr
#End If
    
    Select Case s.Line.BeginArrowheadStyle
    Case msoArrowheadOval
        marker_start = " marker-start=""url(#Circle)"""
        markersNeeded = True
    Case msoArrowheadTriangle
        marker_start = " marker-start=""url(#TriangleS)"""
        markersNeeded = True
    End Select

    Select Case s.Line.EndArrowheadStyle
    Case msoArrowheadOval
        marker_end = " marker-end=""url(#Circle)"""
        markersNeeded = True
    Case msoArrowheadTriangle
        marker_end = " marker-end=""url(#TriangleE)"""
        markersNeeded = True
    End Select
    
    polyline_to_svg = dbgStr _
     & "<polyline points=""" & get_points(s, False) & myQ _
     & fillStrokeStyle(s) _
     & marker_start _
     & marker_end _
     & "/>"
     
    If markersNeeded And Not markersDone Then
        outputDefs
        markersDone = True
    End If
End Function    ' polyline_to_svg



Private Function polygon_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If

    polygon_to_svg = dbgStr _
     & "<polygon points=""" & get_points(s, True) & myQ _
     & fillStrokeStyle(s) _
     & "/>"
End Function    ' polygon_to_svg



Private Function rect_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If
    
    rect_to_svg = dbgStr _
     & "<rect " _
     & get_svg_rect_points(s) _
     & fillStrokeStyle(s) _
     & "/>"
End Function    ' rect_to_svg



Private Function rndrect_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim dbgStr As String
    Dim minDimSng As Single
    Dim rxStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If
    minDimSng = s.Width
    If s.Height < minDimSng Then minDimSng = s.Height
    rxStr = "rx=""" & myDecFmt(s.Adjustments(1) * minDimSng) & """ "
    
    rndrect_to_svg = dbgStr _
     & "<rect " _
     & rxStr _
     & get_svg_rect_points(s) _
     & fillStrokeStyle(s) _
     & "/>"
End Function    ' rndrect_to_svg



Private Function ellipse_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim dbgStr As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If
    
    ellipse_to_svg = dbgStr _
     & "<ellipse " _
     & get_svg_ellipse_points(s) _
     & fillStrokeStyle(s) _
     & "/>"
End Function    ' ellipse_to_svg



'
' Returns the path to file gestFullNameStr relative to file hostFullNameStr
' These two /must/ be full file names and not paths!
' /Warning:/ there is no check whether the drive is common!
'
Private Function get_rel_dir(ByVal hostFullNameStr As String, ByVal gestFullNameStr As String, ByVal sepStr As String) As String
    Dim i As Long
    Dim j As Long
    Dim k As Long
    Dim hostDirsStr(16) As String
    Dim gestDirsStr(16) As String
    Dim filOb As Object
    Dim fldOb As Object

    i = -1
    Set filOb = globFSOb.GetFile(hostFullNameStr)
    Set fldOb = filOb.ParentFolder
    While fldOb.Name <> ""
        i = i + 1
        hostDirsStr(i) = fldOb.Name
        Set fldOb = fldOb.ParentFolder
    Wend

    j = -1
    Set filOb = globFSOb.GetFile(gestFullNameStr)
    Set fldOb = filOb.ParentFolder
    While fldOb.Name <> ""
        j = j + 1
        gestDirsStr(j) = fldOb.Name
        Set fldOb = fldOb.ParentFolder
    Wend

    For k = 0 To 16 ' skip common directories
        If i < 0 Then Exit For
        If j < 0 Then Exit For
        If hostDirsStr(i) <> gestDirsStr(j) Then Exit For
        i = i - 1
        j = j - 1
    Next k

    If i < 0 Then   ' when host exhausted
        get_rel_dir = "." & sepStr
        For k = 0 To 16 ' append guest
            If j < 0 Then Exit For
            get_rel_dir = get_rel_dir & gestDirsStr(j) & sepStr
            j = j - 1
        Next k
    Else
        get_rel_dir = ""
        For k = 0 To 16 ' traverse host up
            If i < 0 Then Exit For
            get_rel_dir = get_rel_dir & ".." & sepStr
            i = i - 1
        Next k

        For k = 0 To 16 ' append guest
            If j < 0 Then Exit For
            get_rel_dir = get_rel_dir & gestDirsStr(j) & sepStr
            j = j - 1
        Next k
  
    End If
    
    get_rel_dir = get_rel_dir & globFSOb.GetFileName(gestFullNameStr)

End Function




Private Function linked_pict_to_svg(ByVal s As Shape, ByVal indentStr As String) As String
    Dim dbgStr As String
    Dim srcURL As String
#If MY_DEBUG Then
    dbgStr = indentStr & "<!-- " & s.Name & " -->" & vbLf & indentStr
#End If

Rem It should be a relative path. But this fails when the generated SVG gets inserted in a PPT in turn
Rem srcURL = get_rel_dir(ActivePresentation.FullName, s.LinkFormat.SourceFullName, "/")
Rem ...hence
    srcURL = s.LinkFormat.SourceFullName
    linked_pict_to_svg = dbgStr _
     & "<image " _
     & get_svg_rect_points(s) _
     & " href=" & myQ & srcURL & myQ _
     & fillStrokeStyle(s) _
     & "/>"
End Function    ' linked_pict_to_svg



Private Sub decompose_group(ByVal s As Shape, ByVal indentStr As String)
    Dim sr As Shape
    Dim dbgStr As String
    
#If MY_DEBUG Then
    dbgStr = " <!-- " & s.Name & " -->"
#End If
    strmOb.WriteText indentStr & "<g>" & dbgStr, adWriteLine

    For Each sr In s.GroupItems
        handleShape sr, indentStr & Space(myItN)
    Next sr

    strmOb.WriteText indentStr & "</g>", adWriteLine

End Sub ' decompose_group



Private Function isShapeCurved(ByRef s As Shape) As Boolean
    Dim i As Long
    
    isShapeCurved = False   ' assume it is a polyline

    ' if there is at least one curved, I consider it curved
    For i = 1 To s.Nodes.Count
        If s.Nodes(i).SegmentType = msoSegmentCurve Then
            isShapeCurved = True
            Exit Function
        End If
    Next i
    
End Function



Private Function isShapePolygon(ByRef s As Shape) As Boolean
    
    isShapePolygon = _
    (s.Vertices(1, 1) = s.Vertices(s.Nodes.Count, 1)) And _
    (s.Vertices(1, 2) = s.Vertices(s.Nodes.Count, 2))
End Function




'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Higher level functions
'


Private Sub handleShape(ByRef s As Shape, ByVal indentStr As String)
    
#If 0 Then  ' Rotation not supported yet
    If s.Rotation <> 0 Then
    End If
#End If
    
    Select Case s.Type
    Case msoGroup
        decompose_group s, indentStr
    
    Case msoLine
        strmOb.WriteText line_to_svg(s, indentStr), adWriteLine
    
    Case msoTextBox
        strmOb.WriteText rect_to_svg(s, indentStr), adWriteLine
        If s.TextFrame.HasText Then strmOb.WriteText textbox_to_svg(s, indentStr), adWriteLine
    
    Case msoFreeform
        If isShapePolygon(s) Then
            If isShapeCurved(s) Then
                strmOb.WriteText curve_to_svg(s, indentStr), adWriteLine
            Else
                strmOb.WriteText polygon_to_svg(s, indentStr), adWriteLine
            End If
        Else
            If isShapeCurved(s) Then
                strmOb.WriteText curve_to_svg(s, indentStr), adWriteLine
            Else
                strmOb.WriteText polyline_to_svg(s, indentStr), adWriteLine
            End If
        End If
        If s.TextFrame.HasText Then strmOb.WriteText textbox_to_svg(s, indentStr), adWriteLine
        
    Case msoLinkedPicture
        strmOb.WriteText linked_pict_to_svg(s, indentStr), adWriteLine
    
    Case msoAutoShape
        Select Case s.AutoShapeType
        Case msoShapeRectangle
            strmOb.WriteText rect_to_svg(s, indentStr), adWriteLine
            If s.TextFrame.HasText Then strmOb.WriteText textbox_to_svg(s, indentStr), adWriteLine
            
        Case msoShapeRoundedRectangle
            strmOb.WriteText rndrect_to_svg(s, indentStr), adWriteLine
            If s.TextFrame.HasText Then strmOb.WriteText textbox_to_svg(s, indentStr), adWriteLine
            
        Case msoShapeOval
            strmOb.WriteText ellipse_to_svg(s, indentStr), adWriteLine
            If s.TextFrame.HasText Then strmOb.WriteText textbox_to_svg(s, indentStr), adWriteLine
            
        Case msoShapeMixed
            Rem "line" works for /straight/ connectors.
            Rem For a corner or round connector, it just does a straight line from beginning to end,
            Rem maybe because s.Top, s.Left, s.Width, s.Height indicate the bounding box in this case.
            Rem Connectors are not polylines neither groups
            If s.Connector = msoTrue Then
                strmOb.WriteText line_to_svg(s, indentStr), adWriteLine
            Else
                ' unexptected stuff...
                strmOb.WriteText vbLf & "<!-- " & s.Name & " is msoShapeMixed (-2) but not a Connector -->" & vbLf, adWriteLine
            End If
    
        Case Else
            globErrText = globErrText & "AutoShape >" & s.Name & "< is unhandled type " & s.AutoShapeType & vbNewLine
        End Select  ' s.AutoShapeType
    
    Case msoPlaceholder
        ' I trap out place holders here, but the title one I handle explicitly elsewhere
        ' ("Shapes.Title" or equivalently "Shapes.Placeholders(1)" )
        Rem strmOb.WriteText vbLF & "<!-- " & s.Name & " is msoPlaceholder (14) -->" & vbLF, adWriteLine
    
    Case Else
        globErrText = globErrText & "Shape >" & s.Name & "< is unhandled type " & s.Type & vbNewLine
    End Select  ' s.Type

End Sub ' handleShape




Private Sub outputDefs()
' All the static settings that shall be in the SVG file
'
    Dim it As String
#If MY_DEBUG Then
    it = Space(myItN)
#End If
  
  ''  black markers - this could be made cleverer by dynamically writing the SVG as an arrow'ed object is encountered
  strmOb.WriteText it & "<defs>", adWriteLine
  strmOb.WriteText it & it & "<marker id=""TriangleE""", adWriteLine
  strmOb.WriteText it & it & it & "viewBox=""" & "0 0 8 8" & """ refX=""" & "7" & """ refY=""" & "4" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "markerUnits = ""strokeWidth""", adWriteLine
  strmOb.WriteText it & it & it & "markerWidth=""" & "8" & """ markerHeight=""" & "4" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "orient=""auto"">", adWriteLine
  strmOb.WriteText it & it & it & "<polygon points=""" & "0,0 8,4 0,8" & """ />", adWriteLine
  strmOb.WriteText it & it & "</marker>", adWriteLine
  strmOb.WriteText it & it & "<marker id=""TriangleS""", adWriteLine
  strmOb.WriteText it & it & it & "viewBox=""" & "0 0 8 8" & """ refX=""" & "1" & """ refY=""" & "4" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "markerUnits = ""strokeWidth""", adWriteLine
  strmOb.WriteText it & it & it & "markerWidth=""" & "8" & """ markerHeight=""" & "4" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "orient=""auto"">", adWriteLine
  strmOb.WriteText it & it & it & "<polygon points=""" & "8,0 0,4 8,8" & """ />", adWriteLine
  strmOb.WriteText it & it & "</marker>", adWriteLine
  strmOb.WriteText it & it & "<marker id=""Circle""", adWriteLine
  strmOb.WriteText it & it & it & "viewBox=""" & "0 0 10 10" & """ refX=""" & "5.5" & """ refY=""" & "5" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "markerUnits = ""strokeWidth""", adWriteLine
  strmOb.WriteText it & it & it & "markerWidth=""" & "8" & """ markerHeight=""" & "6" & myQ, adWriteLine
  strmOb.WriteText it & it & it & "orient=""auto"" style=""fill:black;"">", adWriteLine
  strmOb.WriteText it & it & it & "<circle cx=""" & "5" & """ cy=""" & "5" & """ r=""" & "3" & """ />", adWriteLine
  strmOb.WriteText it & it & "</marker>", adWriteLine
  strmOb.WriteText it & "</defs>", adWriteLine
End Sub ' outputDefs



Private Sub myStrmOpen()
    Set strmOb = CreateObject("ADODB.Stream")
    strmOb.Type = adTypeText
    strmOb.Mode = adModeReadWrite   ' adWrite fails with error on SaveToFile
    strmOb.Charset = "utf-8"
    strmOb.LineSeparator = adLF
    strmOb.Open
End Sub


Private Sub myStrmClose(ByRef fn As String)
    On Error Resume Next
    strmOb.SaveToFile fn, adSaveCreateOverWrite
    If Err <> 0 Then
        MsgBox "Export cancelled." & vbNewLine & _
        "Cannot write to location" & vbNewLine & fn, _
        vbExclamation, globModName
        Err = 0
        On Error GoTo 0
    End If
    strmOb.Flush
    strmOb.Close
End Sub




Sub export_to_svg()
' This is the user macro
'
    Dim docPropsOb As DocumentProperties
    Dim fnSVG As String
    Dim currSlideOb As Slide
    Dim slideIndex As Long
    Dim s As Shape
    Dim dbgStr As String
    Dim indentStr As String

    markersDone = False
#If MY_DEBUG Then
    indentStr = Space(myItN)
#End If

    Set globFSOb = CreateObject("Scripting.FileSystemObject")
  
    Set docPropsOb = Application.ActivePresentation.BuiltInDocumentProperties
    On Error Resume Next
    ' cater for the different views avail. in PPT
    slideIndex = ActiveWindow.View.Slide.slideIndex ' but you must select a slide
    If Err <> 0 Then
        MsgBox "Export cancelled." & vbNewLine & _
        "Please select first a slide to export", _
        vbExclamation, globModName
        Err = 0
        On Error GoTo 0
        Exit Sub
    End If
    Set currSlideOb = ActivePresentation.Slides(slideIndex)
    

    ' construct SVG file name
    fnSVG = globFSOb.GetBaseName(ActivePresentation.Name)
    fnSVG = globFSOb.BuildPath(ActivePresentation.Path, _
    fnSVG & "-" & Right("0" & currSlideOb.SlideNumber, 2) & ".svg")
    
    
    myStrmOpen
    strmOb.WriteText "<svg xmlns=""http://www.w3.org/2000/svg""", adWriteChar
#If USE_ABS_VIEW_PORT Then
    strmOb.WriteText " width=""" & ActivePresentation.PageSetup.SlideWidth & "pt""" _
     & " height=""" & ActivePresentation.PageSetup.SlideHeight & "pt""", adWriteChar
#End If
     strmOb.WriteText " viewBox=""0 0 " _
     & myDecFmt(ActivePresentation.PageSetup.SlideWidth) _
     & " " _
     & myDecFmt(ActivePresentation.PageSetup.SlideHeight) _
     & """>", adWriteLine

    ' Meta data
    On Error Resume Next    ' Title may not exist for this slide
    strmOb.WriteText indentStr & "<title>" & normaliseText(currSlideOb.Shapes.Title.TextFrame.TextRange.Text) & "</title>", adWriteLine
    On Error GoTo 0
#If 0 Then
    strmOb.WriteText indentStr & "<desc>" & normaliseText(docPropsOb.Item("Comments")) & "</desc>", adWriteLine
#End If
    
    
    globErrText = ""


    '' Output a rectangle as the slide Background


#If USE_BACKGROUND Then

    If currSlideOb.FollowMasterBackground Then
#If 0 Then  ' this code does not work. The colour is always white
        If currSlideOb.Design.SlideMaster.Background.Fill.Visible Then
            MsgBox "master" _
            & vbNewLine & rgb_to_colour_hash(currSlideOb.Design.SlideMaster.Background.Fill.ForeColor.RGB) _
            & vbNewLine & rgb_to_colour_hash(currSlideOb.Design.SlideMaster.Background.Fill.BackColor.RGB), _
            vbExclamation, globModName
        Else
            MsgBox "master bg invisible", vbExclamation, globModName
        End If
#Else   ' *** provisionally, I always put a white rectangle as background
 #If MY_DEBUG Then
        dbgStr = indentStr & "<!-- White Background -->" & vbLf & indentStr
 #Else
        dbgStr = ""
 #End If
        strmOb.WriteText dbgStr _
         & "<rect x=""0"" y=""0" _
         & """ width=""" & myDecFmt(ActivePresentation.PageSetup.SlideWidth) _
         & """ height=""" & myDecFmt(ActivePresentation.PageSetup.SlideHeight) _
         & """" _
         & " style=""fill:white" _
         & """/>", adWriteLine
#End If


    Else


        If currSlideOb.Background.Fill.Visible Then
#If MY_DEBUG Then
            dbgStr = indentStr & "<!-- Slide Background -->" & vbLf & indentStr
#Else
            dbgStr = ""
#End If
            strmOb.WriteText dbgStr _
             & "<rect x=""0"" y=""0" _
             & """ width=""" & myDecFmt(ActivePresentation.PageSetup.SlideWidth) _
             & """ height=""" & myDecFmt(ActivePresentation.PageSetup.SlideHeight) _
             & """" _
             & " style=""fill:" _
             & rgb_to_colour_hash(currSlideOb.Background.Fill.ForeColor.RGB) _
             & """/>", adWriteLine
        Else
            strmOb.WriteText indentStr & "<!-- Slide Background invisible -->", adWriteLine
        End If

    End If  ' FollowMasterBackground
            
#End If ' USE_BACKGROUND



    For Each s In currSlideOb.Shapes
        handleShape s, indentStr
    Next  ' Shape


    
    strmOb.WriteText "</svg>", adWriteLine
    myStrmClose fnSVG
    If globErrText <> "" Then MsgBox globErrText, vbExclamation, globModName
    
End Sub ' export_to_svg



'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Auxiliary Macros
'


' Enum SegmentType
' msoSegmentLine    = 0
' msoSegmentCurve   = 1

' Enum EditingType
' msoEditingAuto        = 0
' msoEditingCorner      = 1
' msoEditingSmooth      = 2
' msoEditingSymmetric   = 3



Sub showSegments()
' For a /selected/ shape, show the segment types
'
    Dim i As Long
    Dim s As String
    
    s = "Seg.Type" & vbTab & "Edt.Type" & vbNewLine
    With ActiveWindow.Selection.ShapeRange
        For i = 1 To .Nodes.Count
            s = s & .Nodes(i).SegmentType & vbTab
            On Error Resume Next
            s = s & .Nodes(i).EditingType & vbNewLine
            If Err <> 0 Then
                s = s & "unavail" & vbNewLine
                Err = 0
            End If
            On Error GoTo 0
        Next i
    End With
    
    MsgBox s, vbInformation, globModName
End Sub



Sub setSegmentsAsLines()
' For a /selected/ shape, force straight line segments
'
    Dim i As Long
    
    With ActiveWindow.Selection.ShapeRange.Nodes
        i = 1   ' while acc. to MS docu.
        ' for does not work because the number of nodes changes!
        While i <= .Count
            .SetSegmentType i, msoSegmentLine
            i = i + 1
        Wend
    End With
End Sub




'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' To access the macros as a Powerpoint "Add-In" (.ppam file)
'
'
' Based on code from:
' https://www.rdpslides.com/pptfaq/FAQ00031_Create_an_ADD-IN_with_TOOLBARS_that_run_macros.htm
'
Sub Auto_Open()
    Dim oToolbar As CommandBar

    On Error Resume Next

    ' Create the toolbar. PowerPoint will throw an error if it already exists
    Set oToolbar = CommandBars.Add(Name:="trppt2svg Tool Bar", Position:=msoBarFloating, Temporary:=True)
    If Err.Number <> 0 Then
        Exit Sub
    End If

    On Error GoTo ErrorHandler
    
    With oToolbar.Controls.Add(Type:=msoControlButton)
         .DescriptionText = "Description for developer"
         .Caption = "Export To SVG"     ' Tooltip
         .OnAction = "export_to_svg"
'         .Style = msoButtonIcon ' Button displays as icon, not text or both
         .Style = msoButtonIconAndCaptionBelow
         .FaceId = 43   ' chooses icon from the available MSOffice icons. See
         ' https://excelrocks.com/working-with-faceid-in-excel/
    End With
    
    With oToolbar.Controls.Add(Type:=msoControlButton)
         .DescriptionText = "Description for developer"
         .Caption = "Show Segment"
         .OnAction = "showSegments"
         .Style = msoButtonIconAndCaptionBelow
         .FaceId = 43
    End With
    
    With oToolbar.Controls.Add(Type:=msoControlButton)
         .DescriptionText = "Description for developer"
         .Caption = "Set Segments as Lines"
         .OnAction = "setSegmentsAsLines"
         .Style = msoButtonIconAndCaptionBelow
         .FaceId = 43
    End With
         
    oToolbar.Visible = True
    Exit Sub


ErrorHandler:
    MsgBox Err.Number & vbNewLine & Err.Description
    Exit Sub
End Sub




'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
